﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;
using System.IO.Ports;
using System.Windows.Forms;


namespace iceFinder
{
    public partial class Form1 : Form
    {
        // Create instances of the iver2 variables for communicating with main cpu
        SerialPort _port;
        //BackgroundWorker bg;
        FileInfo fLOG;
        VehicleState vehiState;
        CompassState compState;
        PowerState powerState;
        ackMessage ackMsg;
        string askForData = "$OSD,C,G,P,S,Y*2A\r\n";
        Thread t;

        // The control variables
        double motorSpeed = 128, yawFin = 128, pitchFin = 128;

        // The main constructor
        public Form1()
        {
            InitializeComponent();
            startButton.Enabled = true;
            stopButton.Enabled = false;
        }


        public void startRemote()
        {
            while (true)
            {
                Thread.Sleep(1000);
            }
        }

        private void startButton_Click(object sender, EventArgs e)
        {
            try
            {
                // Enable the stop button and disable the start button
                startButton.Enabled = false;
                stopButton.Enabled = true;

                t = new Thread(new ThreadStart(RunThread));
                t.Start();
                LogBox.Items.Add("Start Button: Started!");
            }
            catch (Exception f)
            {
                LogBox.Items.Add("Start Button Initialization Error: " + f.Message);
            }

        }
        private void stopButton_Click(object sender, EventArgs e)
        {
            try
            {
                t.Abort();
                _port.Close();
                startButton.Enabled = true;
                stopButton.Enabled = false;
                LogBox.Items.Add("Stop Button: Stopped!");

            }
            catch (Exception f)
            {
                LogBox.Items.Add("Stop Button: " + f.Message);
            }
        }


        void RunThread()
        {
            //Initialization
            fLOG = new FileInfo(@"\\IVER2-66\Logs\iceFinderlogs\iceFinderLog.txt");
            StreamWriter w = fLOG.AppendText();

            // Print out the program is running to console
            LogBox.Items.Add("Running!!!");

            // Open up the serial port to communicate with main cpu
            _port = new SerialPort("COM2", 19200, Parity.None, 8, StopBits.One);
            _port.Handshake = Handshake.None;
            _port.DataReceived += new SerialDataReceivedEventHandler(sp_DataReceived);
        
            // Print out the program has created the serial port connection
            LogBox.Items.Add("Created Port!");

            // Create primitive control with the com port
            primitiveControl primControl = new primitiveControl(_port);
            
            // Wait for 5 seconds before starting a mission
            sendCommand(askForData);
            Thread.Sleep(5000);

            // This loop will be helpful for simple primitive control stuff
            try
            {
                //Begin control
                while (true)
                {
                    // Get Sensor Data 
                    sendCommand(askForData);

                    // Wait for 0.5 seconds to ensure all data is updated
                    Thread.Sleep(500);

                    // Set actuator control signals 
                    // Note 1: a value of 128 will zero the control effort
                    // Note 2: a value of 0 will be max in one direction
                    // Note 3: a value of 255 will be a max in the other direction
                    yawFin = 128;
                    pitchFin = 128;
                    motorSpeed = 128;
                    double lat = vehiState.getLatitude();
                    double lon = vehiState.getLongitude();
                    LogBox.Items.Add(" New data "+lat +" "+lon);

                    // Control the actuators
                    ControlActuators(primControl);

                }
            }
            catch (Exception f)
            {
                LogBox.Items.Add("BG Run Failed..." + f.Message);
            }


        }

         

        // Update yaw, pitch, and speed 
        private void ControlActuators(primitiveControl pC)
        {
            if (compState != null){
                
                // Send the controls
                pC.setBottomFin((int)yawFin);
                pC.setStarboardFin((int)pitchFin);
                pC.setPortFin((int)pitchFin);
                pC.setMotorSpeed((int)motorSpeed);
                String err = pC.sendControl();
            }
        }



        private void LogUpdates(StreamWriter w)
        {
            if (compState != null)
            {
                LogBox.Items.Add("Yaw Fin:" + (int)yawFin + ", Pitch Fin:" + (int)pitchFin + ", Motor:" + (int)motorSpeed);
            } 
            else
                LogBox.Items.Add("Mission Update: No compState still ");
        }




        private void LogBox_SelectedIndexChanged(object sender, EventArgs e)
        {

        }


        
  
        void sp_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {

            try
            {
                string data = _port.ReadLine();


                if (data.StartsWith("$OSI"))
                {

                    vehiState = new VehicleState(data);
                }
                else if (data.StartsWith("$C"))
                {
                    compState = new CompassState(data);
                }
                else if (data.StartsWith("$OPI"))
                {
                    powerState = new PowerState(data);
                }
                else if (data.StartsWith("$ACK"))
                {
                    ackMsg = new ackMessage(data);
                }     
            }
            catch (Exception f)
            {
                LogBox.Items.Add("Receive from COM Port Failed!!!" + f.Message);
            }
            LogBox.SelectedIndex = LogBox.Items.Count - 1;

        }

        
        void sendCommand(string command)
        {
            try
            {
                if (!(_port.IsOpen))
                {
                    _port.Open();
                }
                _port.Write(command);
            }
            catch (Exception e)
            {
                LogBox.Items.Add("Send Command to COM Port Failed!!!" + e.Message);
            }
            LogBox.SelectedIndex = LogBox.Items.Count - 1;
        }

     
    }
}