﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace iceFinder
{
    class VehicleState
    {
        int topFin; //fin positions 0x80 or 128 is neutral  
        int botFin;
        int portFin;
        int starFin;
        int motor; //motor speed, 0 is full reverse, 255 full forward
        string mode; // N = Normal UVC control, S = stopped, no mission running, M = manual override, P = manual park, A = override autopilots
        int nextWP; // Waypoint vehicle is navigating towards
        double latitude; // Current estimated latitude using GPS and dead reckoning
        double longitude; // Current estimated longitude using GPS and dead reckoning   
        double speedKnots; // Estimated speed
        double distNextWP; //in nautical miles
        string errorState; // N = no errors, E = battery below 10%, M = mechanical problem, servo unresponsive
                           // Sop = over pitch, Stl = exceeded time limit, Sfp = no forward progress, Sed = exceeded max depth,
                           // Sup = no upward progress, Stf = safety tow float engaged, Srp = safety return path engaged and SRP mission started
        double altimeter; //in feet
        int parkTime; //in seconds


        public VehicleState(string data)
        {

            string[] splitData;

            splitData = data.Split(new char[] { ',' });

            if (splitData.Length == 10)
            {
                string[] messageEnd = splitData[9].Split(new char[] { '*' });

                topFin = int.Parse(splitData[1].Substring(0, 2), System.Globalization.NumberStyles.HexNumber);
                botFin = int.Parse(splitData[1].Substring(2, 2), System.Globalization.NumberStyles.HexNumber);
                portFin = int.Parse(splitData[1].Substring(4, 2), System.Globalization.NumberStyles.HexNumber);
                starFin = int.Parse(splitData[1].Substring(6, 2), System.Globalization.NumberStyles.HexNumber);
                motor = int.Parse(splitData[1].Substring(8, 2), System.Globalization.NumberStyles.HexNumber);

                mode = splitData[2];
                nextWP = Convert.ToInt32(splitData[3]);
                latitude = Convert.ToDouble(splitData[4]);
                longitude = Convert.ToDouble(splitData[5]);
                speedKnots = Convert.ToDouble(splitData[6]);
                distNextWP = Convert.ToDouble(splitData[7]);
                errorState = splitData[8];
                altimeter = Convert.ToDouble(messageEnd[0]);
            }

            else if (splitData.Length == 11)
            {
                string[] messageEnd = splitData[10].Split(new char[] { '*' });

                topFin = int.Parse(splitData[1].Substring(0, 2), System.Globalization.NumberStyles.HexNumber);
                botFin = int.Parse(splitData[1].Substring(2, 2), System.Globalization.NumberStyles.HexNumber);
                portFin = int.Parse(splitData[1].Substring(4, 2), System.Globalization.NumberStyles.HexNumber);
                starFin = int.Parse(splitData[1].Substring(6, 2), System.Globalization.NumberStyles.HexNumber);
                motor = int.Parse(splitData[1].Substring(8, 2), System.Globalization.NumberStyles.HexNumber);

                mode = splitData[2];
                nextWP = Convert.ToInt32(splitData[3]);
                latitude = Convert.ToDouble(splitData[4]);
                longitude = Convert.ToDouble(splitData[5]);
                speedKnots = Convert.ToDouble(splitData[6]);
                distNextWP = Convert.ToDouble(splitData[7]);
                errorState = splitData[8];
                altimeter = Convert.ToDouble(splitData[9]);
                parkTime = Convert.ToInt32(messageEnd[0]);
            }
            else if (splitData.Length == 12) 
            {
                string[] messageEnd = splitData[11].Split(new char[] { '*' });

                topFin = int.Parse(splitData[1].Substring(0, 2), System.Globalization.NumberStyles.HexNumber);
                botFin = int.Parse(splitData[1].Substring(2, 2), System.Globalization.NumberStyles.HexNumber);
                portFin = int.Parse(splitData[1].Substring(4, 2), System.Globalization.NumberStyles.HexNumber);
                starFin = int.Parse(splitData[1].Substring(6, 2), System.Globalization.NumberStyles.HexNumber);
                motor = int.Parse(splitData[1].Substring(8, 2), System.Globalization.NumberStyles.HexNumber);

                mode = splitData[2];
                nextWP = Convert.ToInt32(splitData[3]);
                latitude = Convert.ToDouble(splitData[4]);
                longitude = Convert.ToDouble(splitData[5]);
                speedKnots = Convert.ToDouble(splitData[6]);
                distNextWP = Convert.ToDouble(splitData[7]);
                errorState = splitData[8];
                altimeter = Convert.ToDouble(splitData[9]);
                parkTime = Convert.ToInt32(messageEnd[0]);
            }
            else
            {
                latitude = 99.0;
                botFin = splitData.Length;
            }
        }

        public override string ToString()
        {
            return "Top Fin: " + topFin + " Bot Fin: " + botFin +
                   " Port Fin: " + portFin + " Star Fin: " + starFin + 
                   " Motor: " + motor + " Mode: " + mode + 
                   " Next WP: " + nextWP + " Lat: " + latitude + 
                   " Longitude: " + longitude + " Speed(knots): " + speedKnots + 
                   " Dist to next WP: " + distNextWP + " Error State: " + errorState + 
                   " Altimeter: " + altimeter;
        }

        public string logString()
        {
            return topFin.ToString() + ";" + botFin.ToString() + ";" + portFin.ToString() + ";" +
                   starFin.ToString() + ";" + motor.ToString() + ";" + mode + ";" + nextWP.ToString()
                   + ";" + latitude.ToString() + ";" + longitude.ToString() + ";" + speedKnots.ToString()
                   + ";" + distNextWP.ToString() + ";" + errorState.ToString() + ";" + altimeter.ToString()
                   + ";" + parkTime.ToString() + ";";
        }

        public int getTopFin()
        {
            return topFin;
        }

        public int getBotFin()
        {
            return botFin;
        }

        public int getPortFin()
        {
            return portFin;
        }

        public int getStarFin()
        {
            return starFin;
        }

        public int getMotor()
        {
            return motor;
        }

        public string getMode()
        {
            return mode;
        }

        public int getNextWP()
        {
            return nextWP;
        }

        public double getLatitude()
        {
            return latitude;
        }

        public double getLongitude()
        {
            return longitude;
        }

        public double getSpeed()
        {
            return speedKnots;
        }

        public double getDistNextWP()
        {
            return distNextWP;
        }

        public string getErrorState()
        {
            return errorState;
        }

        public double getAltimeter()
        {
            return altimeter;
        }

        /*       public int getParkTime()
               {
                   return parkTime;
               }*/
    }
}
