﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;

namespace iceFinder
{
    class parkMessage
    {
        double latitude;
        double longitude;
        int parkTime; //minutes
        int speed; //knots
        SerialPort _aPort;
        string errorState;

        public parkMessage(double lat, double lon, int time, int spd, SerialPort _port)
        {
            latitude = lat;
            longitude = lon;
            parkTime = time;
            speed = spd;
            _aPort = _port;
        }


        public string sendControl()
        {
            sendCommand(this.commandMessage(), _aPort);
            return errorState;
        }

        public void setAll(double lat, double lon, int time, int spd)
        {
            latitude = lat;
            longitude = lon;
            parkTime = time;
            speed = spd;
        }
        public void setParkLat(double lat)
        {
            latitude = lat;
        }
        public void setParkLong(double lon)
        {
            longitude = lon;
        }
        public void setParkTime(int time)
        {
            parkTime = time;
        }
        public void setParkSpeed(int spd)
        {
            speed = spd;
        }
        public void setPort(SerialPort _port)
        {
            _aPort = _port;
        }

        public double getParkLat()
        {
            return latitude;
        }
        public double getParkLong()
        {
            return longitude;
        }
        public int getParkTime()
        {
            return parkTime;
        }
        public int getParkSpeed()
        {
            return speed;
        }
        public SerialPort getPort()
        {
            return _aPort;
        }

        private void sendCommand(string command, SerialPort _port)
        {
            try
            {
                if (!(_port.IsOpen))
                {
                    _port.Open();
                }
                _port.Write(command);
                errorState = "noError";
            }
            catch (Exception e)
            {
                errorState = "Park Message Error: " + e.Message;
            }
        }
        private string commandMessage()
        {
            string message;
            string cSum;

            message = "$OPK," + latitude.ToString() + "," + longitude.ToString() + "," +
                      parkTime.ToString() + "," + speed.ToString() + "*";
            cSum = getCheckSum(message);
            message += cSum.Substring(1, 1) + cSum.Substring(2, 1) + "\r\n";
            return message;
        }
        private string getCheckSum(string com)
        {
            int checksum = 0;

            foreach (char c in com)
            {
                if (c == '$')
                {
                    //Ignore
                }
                else if (c == '*')
                {
                    break;
                }
                else
                {
                    if (checksum == 0)
                    {
                        checksum = Convert.ToByte(c);
                    }
                    else
                    {
                        checksum = checksum ^ Convert.ToByte(c);
                    }
                }
            }

            //string str = System.Uri.HexEscape((char)64);

            return System.Uri.HexEscape((char)checksum);
        }
    }
}
