﻿
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Threading;
using System.IO.Ports;
using System.Windows.Forms;


namespace iceFinder
{
    public partial class Form1 : Form
    {
        // Create instances of the iver2 variables for communicating with main cpu
        SerialPort _port;
        //BackgroundWorker bg;
        FileInfo fLOG;
        VehicleState vehiState;
        CompassState compState;
        PowerState powerState;
        ackMessage ackMsg;
        string askForData = "$OSD,C,G,P,S,Y*2A\r\n";
        Thread t;
        double[] desiredLong = {-120.657939,-120.66282,-120.657939 }; // x axis
        double[] desiredLat = {35.299558, 35.300346, 35.299558 }; // Y axis
        double currentLong = -120.662348;
        double currentLat = 35.30046;
        double currentHeading = 0;
        double desiredHeading = 0;

        // The control variables
        double motorSpeed = 128, yawFin = 128, pitchFin = 128;

        // The main constructor
        public Form1()
        {
            InitializeComponent();
            startButton.Enabled = true;
            stopButton.Enabled = false;
        }


        public void startRemote()
        {
            while (true)
            {
                Thread.Sleep(1000);
            }
        }

        private void startButton_Click(object sender, EventArgs e)
        {
            try
            {
                // Enable the stop button and disable the start button
                startButton.Enabled = false;
                stopButton.Enabled = true;

                t = new Thread(new ThreadStart(RunThread));
                t.Start();
                LogBox.Items.Add("Start Button: Started!");
            }
            catch (Exception f)
            {
                LogBox.Items.Add("Start Button Initialization Error: " + f.Message);
            }

        }
        private void stopButton_Click(object sender, EventArgs e)
        {
            try
            {
                t.Abort();
                _port.Close();
                startButton.Enabled = true;
                stopButton.Enabled = false;
                LogBox.Items.Add("Stop Button: Stopped!");

            }
            catch (Exception f)
            {
                LogBox.Items.Add("Stop Button: " + f.Message);
            }
        }


        void RunThread()
        {
            //Initialization
            //fLOG = new FileInfo(@"\\IVER2-66\Logs\iceFinderlogs\iceFinderLog.txt");
            //StreamWriter w = fLOG.AppendText();

            // Print out the program is running to console
            //LogBox.Items.Add("Running!!!");
            AddToLogBox("Running!!!");





            // Open up the serial port to communicate with main cpu
            _port = new SerialPort("COM13", 19200, Parity.None, 8, StopBits.One);
            _port.Handshake = Handshake.None;
            _port.DataReceived += new SerialDataReceivedEventHandler(sp_DataReceived);
        
            // Print out the program has created the serial port connection
            AddToLogBox("Created Simulated!");

            // Create primitive control with the com port
            primitiveControl primControl = new primitiveControl(_port);
            
            // Wait for 5 seconds before starting a mission
            sendCommand(askForData);
            Thread.Sleep(5000);


            // This loop will be helpful for simple primitive control stuff
            try
            {
                DateTime startTime = DateTime.Now;
                int desPointIndex = 0;
                int counter = 0;

                //Begin control
                while (true)
                {
                    // Get Sensor Data 
                    sendCommand(askForData);

                    // Wait for 0.5 seconds to ensure all data is updated
                    Thread.Sleep(1000);


                    // Get localMeasurements
                    currentLat = vehiState.getLatitude();
                    currentLong = vehiState.getLongitude();
                    currentHeading = compState.getHeading();


                    // Add your code here to set yawFin!!!!!!!!!!!!!!!!!!!!!!!!
                    yawFin = 128;
                    motorSpeed = 128;


                    AddToLogBox(counter+" Desired Point Index = " + desPointIndex + ", desiredHeading = " +desiredHeading + ", Current Heading = "+ currentHeading+", yawFin = "+yawFin);

                    // Control the actuators
                    ControlActuators(primControl);

                }
            }
            catch (Exception f)
            {
                AddToLogBox("BG Run Failed..." + f.Message);
            }


        }

         

        // Update yaw, pitch, and speed 
        private void ControlActuators(primitiveControl pC)
        {
            if (compState != null){
                
                // Send the controls
                pC.setTopFin((int)yawFin);
                pC.setBottomFin((int)yawFin);
                pC.setStarboardFin((int)128);
                pC.setPortFin((int)128);
                pC.setMotorSpeed((int)motorSpeed);
                String err = pC.sendControl();
            }
        }


        private void AddToLogBox(string s)
        {
            if (LogBox.InvokeRequired)
                LogBox.BeginInvoke(new MethodInvoker(delegate() { LogBox.Items.Add(s); }));// WriteLogFromThread(msgtype, Message); }));
            else
                LogBox.Items.Add(s);// WtfShared.WriteLog(msgtype, Message);

        }
        private double simulateHeading()
        {
            return currentHeading + (yawFin - 128) * 0.2;
        }

        private double simulateLong()
        {
            return -120.662348;
        }

        private double simulateLat()
        {
            return 35.30046;
        }

        private void LogUpdates(StreamWriter w)
        {
            if (compState != null)
            {
                LogBox.Items.Add("Yaw Fin:" + (int)yawFin + ", Pitch Fin:" + (int)pitchFin + ", Motor:" + (int)motorSpeed);
            } 
            else
                LogBox.Items.Add("Mission Update: No compState still ");
        }




        private void LogBox_SelectedIndexChanged(object sender, EventArgs e)
        {

        }


        private void UpdateLogBoxCounter(int i)
        {
            if (LogBox.InvokeRequired)
                LogBox.BeginInvoke(new MethodInvoker(delegate() { LogBox.SelectedIndex = LogBox.Items.Count + i; }));
            else
                LogBox.SelectedIndex = LogBox.Items.Count + i;

        }
  
        void sp_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {

            try
            {
                string data = _port.ReadLine();


                if (data.StartsWith("$OSI"))
                {

                    vehiState = new VehicleState(data);
                }
                else if (data.StartsWith("$C"))
                {
                    compState = new CompassState(data);
                }
                else if (data.StartsWith("$OPI"))
                {
                    powerState = new PowerState(data);
                }
                else if (data.StartsWith("$ACK"))
                {
                    ackMsg = new ackMessage(data);
                }     
            }
            catch (Exception f)
            {
                LogBox.Items.Add("Receive from COM Port Failed!!!" + f.Message);
            }
            UpdateLogBoxCounter(-1);

        }

        
        void sendCommand(string command)
        {
            try
            {
                if (!(_port.IsOpen))
                {
                    _port.Open();
                }
                _port.Write(command);
            }
            catch (Exception e)
            {
                AddToLogBox("Send Command to COM Port Failed!!!" + e.Message);
            }
            UpdateLogBoxCounter(-1);
        }

     
    }
}