﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;

namespace iceFinder
{
    class primitiveControl
    {
        string topFin;
        string bottomFin;
        string portFin;
        string starboardFin;
        string motorSpeed; //255 = full forward, 0 = full reverse
        string timeOut;
        SerialPort _aPort;
        string errorState;

        //128 is neutral for all control values
        public primitiveControl(int tFin, int bFin, int pFin, int sFin, int mSpeed, int tOut, SerialPort _port)
        {
            topFin = System.Uri.HexEscape((char)tFin).Substring(1,2); //convert int inputs to hex values
            bottomFin = System.Uri.HexEscape((char)bFin).Substring(1,2);
            portFin = System.Uri.HexEscape((char)pFin).Substring(1,2);
            starboardFin = System.Uri.HexEscape((char)sFin).Substring(1,2);
            motorSpeed = System.Uri.HexEscape((char)mSpeed).Substring(1,2);
            timeOut = tOut.ToString();
            _aPort = _port;
        }
        public primitiveControl(SerialPort _port)
        {
            //default to neutral position for all controls, 128 or 0x80
            topFin = "80"; 
            bottomFin = "80";
            portFin = "80";
            starboardFin = "80";
            motorSpeed = "80";
            timeOut = "3";
            _aPort = _port;
        }

        public string sendControl()
        {
            sendCommand(this.commandMessage(), _aPort);
            return errorState;
        }

        public void setAll(int tFin, int bFin, int pFin, int sFin, int mSpeed, int tOut)
        {
            topFin = System.Uri.HexEscape((char)tFin).Substring(1, 2); //convert int inputs to hex values
            bottomFin = System.Uri.HexEscape((char)bFin).Substring(1, 2);
            portFin = System.Uri.HexEscape((char)pFin).Substring(1, 2);
            starboardFin = System.Uri.HexEscape((char)sFin).Substring(1, 2);
            motorSpeed = System.Uri.HexEscape((char)mSpeed).Substring(1, 2);
            timeOut = tOut.ToString();
        }
        public void setTopFin(int finVal)
        {
            topFin = System.Uri.HexEscape((char)finVal).Substring(1, 2);
        }
        public void setBottomFin(int finVal)
        {
            bottomFin = System.Uri.HexEscape((char)finVal).Substring(1, 2);
        }
        public void setPortFin(int finVal)
        {
            portFin = System.Uri.HexEscape((char)finVal).Substring(1, 2);
        }
        public void setStarboardFin(int finVal)
        {
            starboardFin = System.Uri.HexEscape((char)finVal).Substring(1, 2);
        }
        public void setMotorSpeed(int motorVal)
        {
            motorSpeed = System.Uri.HexEscape((char)motorVal).Substring(1, 2);
        }
        public void setTimeout(int newTime)
        {
            timeOut = newTime.ToString();
        }
        public void setPort(SerialPort _port)
        {
            _aPort = _port;
        }

/* Use vehicle state class to get true status, 
 * use these only to check what the controller is attempting to set!!!*/
        public int getTopFin()
        {
            return int.Parse(topFin.Substring(0, 2), System.Globalization.NumberStyles.HexNumber); 
        }
        public int getBottomFin()
        {
            return int.Parse(bottomFin.Substring(0, 2), System.Globalization.NumberStyles.HexNumber);
        }
        public int getPortFin()
        {
            return int.Parse(portFin.Substring(0, 2), System.Globalization.NumberStyles.HexNumber);
        }
        public int getStarboardFin()
        {
            return int.Parse(starboardFin.Substring(0, 2), System.Globalization.NumberStyles.HexNumber);
        }
        public int getMotorSpeed()
        {
            return int.Parse(motorSpeed.Substring(0, 2), System.Globalization.NumberStyles.HexNumber);
        }
        public int getTimeout()
        {
            return Convert.ToInt16(timeOut);
        }
        public SerialPort getPort()
        {
            return _aPort;
        }

        private void sendCommand(string command, SerialPort _port)
        {
            try
            {
                if (!(_port.IsOpen))
                {
                    _port.Open();
                }
                _port.Write(command);
                errorState = command;// "noError";
            }
            catch (Exception e)
            {
                errorState = "Primitive Control Error: " + e.Message;
            }
            
        }
        private string commandMessage()
        {
            string message;
            string cSum;

            message = "$OMP," + topFin + bottomFin + portFin + starboardFin + motorSpeed + ",00," + timeOut + "*";
            cSum = getCheckSum(message);
            message += cSum.Substring(1, 1) + cSum.Substring(2, 1) + "\r\n";
            return message;
        }
        private string getCheckSum(string com)
        {
            int checksum = 0;

            foreach (char c in com)
            {
                if (c == '$')
                {
                    //Ignore
                }
                else if (c == '*')
                {
                    break;
                }
                else
                {
                    if (checksum == 0)
                    {
                        checksum = Convert.ToByte(c);
                    }
                    else
                    {
                        checksum = checksum ^ Convert.ToByte(c);
                    }
                }
            }

            //string str = System.Uri.HexEscape((char)64);

            return System.Uri.HexEscape((char)checksum);
        }

    }
}
