﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;

namespace iceFinder
{
    class servoControl
    {
        double heading; //set point in degrees from 0.0 to 359.9
        double depth; //feet
        int maxAngle; //max angle (from 0-90 degrees) of dive
        double speed; //knots;
        int timeOut; //time in seconds of command life, up to 120
        SerialPort _aPort; //port for command to be sent to
        string errorState;


        public servoControl(double hding, double dpth, int mAngle, double spd, int tOut, SerialPort _port)
        {
            heading = hding;
            depth = dpth;
            maxAngle = mAngle;
            speed = spd;
            timeOut = tOut;
            _aPort = _port;
        }

        public string sendControl()
        {
            sendCommand(this.commandMessage(), _aPort);
            //Sends above settings to UVC to perform operation
            return errorState;
        }  

        public void setAll(double hding, double dpth, int mAngle, double spd, int tOut)
        {
            heading = hding;
            depth = dpth;
            maxAngle = mAngle;
            speed = spd;
            timeOut = tOut;
        }
        public void setHeading(double hding)
        {
            heading = hding;
        }
        public void setDepth(double dpth)
        {
            depth = dpth;
        }
        public void setMaxAngle(int mAngle)
        {
            maxAngle = mAngle;
        }
        public void setSpeed(double spd)
        {
            speed = spd;
        }
        public void setTimeOut(int tOut)
        {
            timeOut = tOut;
        }
        public void setPort(SerialPort _port)
        {
            _aPort = _port;
        }


        /* These are not the current state of the vehicle, only what
         * has been assigned as the next goal state for the Iver to
         * reach.*/

        public double getHeading()
        {
            return heading;
        }
        public double getDepth()
        {
            return depth;
        }
        public int getMaxAngle()
        {
            return maxAngle;
        }
        public double getSpeed()
        {
            return speed;
        }
        public int getTimeOut()
        {
            return timeOut;
        }
        public SerialPort getPort()
        {
            return _aPort;
        }


        private void sendCommand(string command, SerialPort _port)
        {
            try
            {
                if (!(_port.IsOpen))
                {
                    _port.Open();
                }
                _port.Write(command);
                errorState = "noError";
            }
            catch (Exception e)
            {
                errorState = "Servo Control Error: " + e.Message;
            }


        }
        private string commandMessage()
        {
            string message;
            string cSum;

            message = "$OMS," + heading.ToString() + "," + depth.ToString() + "," + maxAngle.ToString() +
                      "," + speed.ToString() + "," + timeOut.ToString() + "*";
            cSum = getCheckSum(message);
            message += cSum.Substring(1, 1) + cSum.Substring(2, 1) + "\r\n";
            return message;
        }
        private string getCheckSum(string com)
        {
            int checksum = 0;

            foreach (char c in com)
            {
                if (c == '$')
                {
                    //Ignore
                }
                else if (c == '*')
                {
                    break;
                }
                else
                {
                    if (checksum == 0)
                    {
                        checksum = Convert.ToByte(c);
                    }
                    else
                    {
                        checksum = checksum ^ Convert.ToByte(c);
                    }
                }
            }

            //string str = System.Uri.HexEscape((char)64);

            return System.Uri.HexEscape((char)checksum);
        }



    }
}
